/// [Grinder](https://google.github.io/grinder.dart) plug-in dedicated to the synchronization of translations with the [Smartling](https://www.smartling.com) service.
library grinder_smartling;

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:grinder/grinder.dart';
import 'package:meta/meta.dart';
import 'package:http/http.dart' as http;
import 'package:path/path.dart' as path;

part 'src/client.dart';
part 'src/file_type.dart';
part 'src/locale.dart';
part 'src/retrieval_type.dart';
part 'src/task.dart';
part 'src/tasks/download_task.dart';
part 'src/tasks/file_task.dart';
part 'src/tasks/upload_task.dart';

/// Downloads the requested file from [Smartling](https://www.smartling.com), using the given settings:
/// - [filePattern]: the pattern indicating the target path of the downloaded files (e.g. `path/to/i18n/{{locale}}.json`).
/// - [locales]: the list of the target locales to be downloaded.
/// - [fileUri]: the value that uniquely identifies the downloaded file.
/// - [projectId]: the project identifier.
/// - [userId]: the user name associated to the account.
/// - [userSecret]: the identification key associated to the account.
///
/// The download can be customized using the following options:
/// - [includeOriginalStrings]: a value indicating whether to return the original string or an empty string when no translation is available.
/// - [retrievalType]: the desired format for the download. Defaults to [RetrievalType.published].
Future download(String filePattern, List locales, {
  @required String fileUri, @required String projectId, @required String userId, @required String userSecret,
  bool includeOriginalStrings = false, String retrievalType = RetrievalType.published
}) async {
  var task = new DownloadTask(fileUri, projectId, userId, userSecret)
    ..filePattern = filePattern
    ..includeOriginalStrings = includeOriginalStrings
    ..locales.addAll(locales.map((locale) => locale is Locale ? locale : new Locale(locale.toString())))
    ..retrievalType = retrievalType;

  return task.run();
}

/// Uploads original source content to [Smartling](https://www.smartling.com), using the given settings:
/// - [file]: the path to the file to be uploaded.
/// - [fileUri]: the value that uniquely identifies the downloaded file.
/// - [projectId]: the project identifier.
/// - [userId]: the user name associated to the account.
/// - [userSecret]: the identification key associated to the account.
///
/// The upload can be customized using the following options:
/// - [authorize]: a value indicating whether content in the file is authorized (available for translation) in all locales.
/// - [callbackUrl]: the URL of the callback called when the file is 100% published for a locale.
/// - [fileType]: the desired format for the download.
Future upload(dynamic file, {
  @required String fileUri, @required String projectId, @required String userId, @required String userSecret,
  bool authorize = false, dynamic callbackUrl, String fileType = ''
}) async {
  var task = new UploadTask(fileUri, projectId, userId, userSecret)
    ..authorize = authorize
    ..file = new FilePath(file).asFile
    ..fileType = fileType;

  if (callbackUrl != null) task.callbackUrl = callbackUrl is Uri ? callbackUrl : Uri.parse(callbackUrl.toString());
  return task.run();
}
