part of grinder_smartling;

/// Provides a HTTP client for communicating with the [Smartling](https://www.smartling.com) service.
class Client {

  /// The URL of the API end point.
  static final Uri endPoint = Uri.parse('https://api.smartling.com');

  /// The identification key associated to the account.
  String _accessToken = '';

  /// Creates a new client.
  Client([this.userId = '', this.userSecret = '']);

  /// The user name associated to the account.
  String userId;

  /// The identification key associated to the account.
  String userSecret;

  /// Authenticates the user, and returns the access token that can be attached to the header of any API request.
  /// Throws a [ArgumentError] if the user credentials are invalid, a [HttpException] if an error occurred while fetching the server response.
  Future<String> authenticate() async {
    if (userId == null || userId.isEmpty) throw new ArgumentError('The user identifier is empty.');
    if (userSecret == null || userSecret.isEmpty) throw new ArgumentError('The user secret is empty.');

    try {
      var body = JSON.encode({
        'userIdentifier': userId,
        'userSecret': userSecret
      });

      var data = _parseResponse(await http.post(
        endPoint.resolve('auth-api/v2/authenticate'),
        body: body,
        headers: {HttpHeaders.CONTENT_TYPE: 'application/json'}
      ));

      if (data is! Map<String, dynamic> || data['accessToken'] is! String)
        throw new FormatException('Invalid response from the Smartling service.', data);

      return data['accessToken'];
    }

    on FormatException catch (e) {
      throw new HttpException(e.message);
    }
  }

  /// Downloads the requested file from [Smartling](https://www.smartling.com), using the given settings:
  /// - [projectId]: the project identifier.
  /// - [fileUri]: the value that uniquely identifies the downloaded file.
  /// - [locale]: the locale identifier as specified in project setup.
  /// - [params]: the additional request parameters.
  Future<List<int>> downloadFile(String projectId, String fileUri, Locale locale, {Map<String, String> params}) async {
    await _checkAccess();

    var query = params ?? {};
    query['fileUri'] = fileUri;

    var url = endPoint
      .resolve('files-api/v2/projects/$projectId/locales/$locale/file')
      .replace(queryParameters: query);

    return http.readBytes(url, headers: {'Authorization': 'Bearer $_accessToken'});
  }

  /// Uploads original source content to [Smartling](https://www.smartling.com), using the given settings:
  /// - [projectId]: the project identifier.
  /// - [fileUri]: the value that uniquely identifies the downloaded file.
  /// - [sourcePath]: the path to the file to be uploaded.
  /// - [params]: the additional request parameters.
  Future uploadFile(String projectId, String fileUri, File sourcePath, {Map<String, String> params}) async {
    await _checkAccess();

    try {
      var request = new http.MultipartRequest('POST', endPoint.resolve('files-api/v2/projects/$projectId/file'))
        ..files.add(new http.MultipartFile.fromBytes('file', await sourcePath.readAsBytes(), filename: path.basename(sourcePath.path)))
        ..fields['fileUri'] = fileUri
        ..fields.addAll(params)
        ..headers['Authorization'] = 'Bearer $_accessToken';

      return _parseResponse(await http.Response.fromStream(await request.send()));
    }

    on FormatException catch (e) {
      throw new HttpException(e.message);
    }
  }

  /// Checks the privilege of the current user.
  /// Returns the authentication token if the user has access.
  Future<String> _checkAccess() async {
    if (_accessToken == null || _accessToken.isEmpty) _accessToken = await authenticate();
    return _accessToken;
  }

  /// Parses and extract the custom data from the specified Smartling [response].
  /// Throws a [FormatException] if the server response is invalid.
  dynamic _parseResponse(http.Response response) {
    if (response.statusCode < 200 || response.statusCode >= 300)
      throw new FormatException('Invalid HTTP response from the Smartling service.', response);

    var body = JSON.decode(response.body);
    if (body is! Map<String, dynamic> || body['response'] is! Map<String, dynamic> || !body['response'].containsKey('data'))
      throw new FormatException('Invalid body response from the Smartling service.', body);

    return body['response']['data'];
  }
}
